#include "key_functions.h"


/**
	Sendet ein Byte an alle Tasten gleichzeitig
	Datenrate 2MBAUD bei 16Mhz

	Bei Benutzung darauf achten dass in der Zeit
	keine anderen Daten	gesendet werden duerfen.

	@param data	Zu sendendes Datenbyte
**/
void sendByteToAll(uint8_t data)
{
	asm volatile (
		"LSL %0 \n"					//Erstes Bit in Carry Shiften
		"LDI r16, 0xFF \n"			//register fuer einen High - Pegel am Ausgang
		"SBC __tmp_reg__, __tmp_reg__ \n"	//tmp mit Carry fuellen
		"STS %2, __tmp_reg__ \n"	//Erstes Bit ausgeben
		"LDI r17, 7 \n"				//Zaehler
	"0:\n"


		"STS %1, __zero_reg__ \n"	//Fallende Taktflanke erzeugen
		"LSL %0 \n"					//Naechstes Bit ins Carry Holen
		"SBC __tmp_reg__, __tmp_reg__ \n"	//tmp mit Carry fuellen
		"DEC r17 \n"				//Zaehler fuer die Bits vermindern
		"STS %1, r16 \n"			//Steigende Taktflanke erzeugen


		"STS %2, __tmp_reg__ \n"	//Aktuelles Bit schreiben
		"BRNE 0b \n"				//Zurueckspringen fuer naechstes Bit

		//Letztes Bit
		"NOP \n"
		"STS %1, __zero_reg__ \n"	//Fallende Taktflanke erzeugen
		"NOP \n"
		"NOP \n"
		"NOP \n"
		"STS %1, r16 \n"			//Steigende Taktflanke erzeugen
		"STS %2, r16 \n"			// Data - Pegel wieder auf HIGH setzen

		:"+d" (data): "M" (_SFR_IO_ADDR(PORT_LC_Clock) + 0x20), "M" (_SFR_IO_ADDR(PORT_LC_Data) + 0x20)
		:"r16", "r17"

	);
}


/**
	Sendet ein Byte an alle Tasten die der Zeilen- und der Spalten-Maske
	entsprechen.

	Bsp:	rowMask = 0b00100010; columnMask = 0b00100101;

			0 ---- 7	Daten werden an die mit '*' gekennzeichneten
		  7	........	Tasten gesendet.
			........
		  |	*.*..*..
		  |	........
		  |	........
		  |	........
			*.*..*..
		  0	........

	Bei Benutzung darauf achten dass in der Zeit
	keine anderen Daten	gesendet werden duerfen.

	@param columnMask	Maske fuer die Spalten.
	@param rowMask		Maske fuer die Zeilen.
	@param data			Zu sendendes Datenbyte.
**/
void sendByteToKeys(uint8_t columnMask, uint8_t rowMask, uint8_t data)
{
	columnMask =~ columnMask;		//Spaltenmaske Negieren (Clock)
	rowMask =~ rowMask;				//Zeilenmaske Negieren (Data)

	asm volatile (

		"LSL %0 \n"					//Erstes Bit in Carry Shiften
		"SBC __tmp_reg__, __tmp_reg__ \n"	//tmp mit Carry fuellen
		"OR __tmp_reg__, %1 \n"		//Data von anderen Tasten auf HIGH
		"STS %4, __tmp_reg__ \n"	//Erstes Bit ausgeben

		"LDI r16, 0xFF \n"			//register fuer einen High - Pegel am Ausgang
		"LDI r17, 7 \n"				//Zaehler
	"0:\n"


		"STS %3, %2 \n"				//Fallende Taktflanke erzeugen, fuer Maske
		"LSL %0 \n"					//Naechstes Bit ins Carry Holen
		"SBC __tmp_reg__, __tmp_reg__ \n"	//tmp mit Carry fuellen
		"OR __tmp_reg__, %1 \n"		//Data von anderen Tasten auf HIGH
		"DEC r17 \n"				//Zaehler fuer die Bits vermindern
		"STS %3, r16 \n"			//Steigende Taktflanke erzeugen



		"STS %4, __tmp_reg__ \n"	//Aktuelles Bit schreiben
		"BRNE 0b \n"

		"NOP \n"
		"STS %3, %2 \n"				//Fallende Taktflanke erzeugen, fuer Maske
		"NOP \n"
		"NOP \n"
		"NOP \n"

		"STS %3, r16 \n"			//Clock - Pegel wieder auf HIGH setzen
		"STS %4, r16 \n"			//Data - Pegel wieder auf HIGH setzen

		:"+d" (data): "r" (columnMask), "r" (rowMask),
		"M" (_SFR_IO_ADDR(PORT_LC_Clock) + 0x20),
		"M" (_SFR_IO_ADDR(PORT_LC_Data) + 0x20)
		:"r17", "r16"

	);

}


/**
	Sendet ein Byte an die Taste mit der Nummer 'keyNumber'.

	Nummerierung der Tasten erfolgt von
	unten, links Taste 0 nach
	oben, rechts Taste 63.

	Bei Benutzung darauf achten dass in der Zeit
	keine anderen Daten	gesendet werden duerfen.

	@param keyNumber	Nummer der Taste von 0 bis 63.
	@param data			Zu sendendes Datenbyte.
**/
void sendByteToKey(uint8_t keyNumber, uint8_t data)
{
	sendByteToKeys((1 << (keyNumber >> 3)), (1 << (keyNumber & 0b00000111)), data);
}

/**
	Sendet ein Byte an die Zeile 'rowNumber'.
	Zeilennummern werden von unten nach oben gezhlt.

	Bei Benutzung darauf achten dass in der Zeit
	keine anderen Daten	gesendet werden duerfen.

	@param rowNumber	Zeilennummer von 0 bis 7.
	@param data			Zu sendendes Datenbyte.
**/
void sendByteToRow(uint8_t rowNumber, uint8_t data)
{
	sendByteToKeys(1 << rowNumber, 0xFF, data);
}


/**
	Sendet ein Byte an die Spalte 'columnNumber'.
	Zpaltennummern werden von links nach rechts gezhlt.

	Bei Benutzung darauf achten dass in der Zeit
	keine anderen Daten	gesendet werden duerfen.

	@param columnNumber	Spaltennummer von 0 bis 7.
	@param data			Zu sendendes Datenbyte.
**/
void sendByteToColumn(uint8_t columnNumber, uint8_t data)
{
	sendByteToKeys(0xFF, 1 << columnNumber, data);
}
